#!/bin/sh
#
# This script will download the archives from the following URLs if they
# are not already downloaded.  Add any mirrors you have to the top of
# this list to override others.
#
# NOTE: The targets below were just taken at random from the mirrors lists.
#
KEYSERVER=pgp.mit.edu
ARCHIVES='
### OpenSSL
# US
http://mirrors.usc.edu/pub/openssl/source/openssl-0.9.8g.tar.gz
http://mirrors.usc.edu/pub/openssl/source/openssl-0.9.8g.tar.gz.asc
ftp://mirrors.usc.edu/pub/openssl/source/openssl-0.9.8g.tar.gz
ftp://mirrors.usc.edu/pub/openssl/source/openssl-0.9.8g.tar.gz.asc
# UK
ftp://ftp.mirror.ac.uk/sites/ftp.openssl.org/source/openssl-0.9.8g.tar.gz
ftp://ftp.mirror.ac.uk/sites/ftp.openssl.org/source/openssl-0.9.8g.tar.gz.asc
# Main site
http://www.openssl.org/source/openssl-0.9.8g.tar.gz
http://www.openssl.org/source/openssl-0.9.8g.tar.gz.asc

### PCRE
# Virginia, USA
http://superb-east.dl.sourceforge.net/sourceforge/pcre/pcre-7.6.tar.gz
http://superb-east.dl.sourceforge.net/sourceforge/pcre/pcre-7.6.tar.gz.sig
# California, USA
http://internap.dl.sourceforge.net/sourceforge/pcre/pcre-7.6.tar.gz
http://internap.dl.sourceforge.net/sourceforge/pcre/pcre-7.6.tar.gz.sig
# UK
http://kent.dl.sourceforge.net/sourceforge/pcre/pcre-7.6.tar.gz
http://kent.dl.sourceforge.net/sourceforge/pcre/pcre-7.6.tar.gz.sig
# Main site
ftp://ftp.csx.cam.ac.uk/pub/software/programming/pcre/pcre-7.6.tar.gz
ftp://ftp.csx.cam.ac.uk/pub/software/programming/pcre/pcre-7.6.tar.gz.sig

### APR
# USA
http://download.nextag.com/apache/apr/apr-1.2.12.tar.gz
http://apache.oregonstate.edu/apr/apr-1.2.12.tar.gz
# UK
http://mirror.public-internet.co.uk/apache/apr/apr-1.2.12.tar.gz
# Main site
http://www.apache.org/dist/apr/apr-1.2.12.tar.gz
http://www.apache.org/dist/apr/apr-1.2.12.tar.gz.asc

### Curl
# California, USA
http://curl.oc1.mirrors.redwire.net/download/curl-7.17.1.tar.gz
http://curl.oc1.mirrors.redwire.net/download/curl-7.17.1.tar.gz.asc
# Texas, USA
http://curl.mirrors.cyberservers.net/download/curl-7.17.1.tar.gz
http://curl.mirrors.cyberservers.net/download/curl-7.17.1.tar.gz.asc
# UK
http://curl.internet.bs/download/curl-7.17.1.tar.gz
http://curl.internet.bs/download/curl-7.17.1.tar.gz.asc
# Main site
http://curl.haxx.se/download/curl-7.17.1.tar.gz
http://curl.haxx.se/download/curl-7.17.1.tar.gz.asc
'

###########################################################################

### What are we named and where are we located
SCRIPT=`basename $0`
SCRIPTDIR=`echo $0 | sed 's!'/$SCRIPT'$!!'`

### Sanity Check
if [ -z "$SCRIPTDIR" ]; then
  echo "ERROR: Could not determine script dir for $0" 1>&2
  exit 1
fi

### Full dir
case "$SCRIPTDIR" in
/*) ;;
*)
  testdir=`pwd`/$SCRIPTDIR
  if [ ! -f "$testdir/$SCRIPT" ]; then
    testdir=`which $SCRIPT`
    if [ ! -f "$testdir/$SCRIPT" ]; then
      echo "ERROR: Could not determine full path to $SCRIPT" 1>&2
      exit 1
    fi
  fi
  SCRIPTDIR=$testdir
;;
esac

. $SCRIPTDIR/common.sh

ARCHIVESDIR=$SCRIPTDIR/archives

### trim comments and whitespace from ARCHIVES
ARCHIVES=`echo "$ARCHIVES" | grep -v '^ *#'`
ARCHIVES=`echo $ARCHIVES`

checksum() {
  exe=$1
  fn=$2
      # formatted or unformatted
      if ( ! grep ' ' "$ARCHIVESDIR/$fn" > /dev/null 2>&1 ); then
        UPSTREAM_CKSUM=`awk '{print $1}' "$ARCHIVESDIR/$fn"`
        fn=`echo $fn | sed 's!^\(.*\)\.[^.][^.]*!\1!'`
      fi
      if [ -z "$UPSTREAM_CKSUM" ]; then
        $exe -c "$ARCHIVESDIR/$fn" && return 0
      else
        CKSUM=`$exe $ARCHIVESDIR/$fn | sed 's/ .*//'`
        CKSUM=`echo $CKSUM`
        if [ "$UPSTREAM_CKSUM" = "$CKSUM" ]; then
          return 0
        fi
        echo "ERROR: $fn '$UPSTREAM_CKSUM' != '$CKSUM'" 1>&2
      fi
  return 1
}

### Look for pgp or gpg
SEARCH_PATH=/opt/sfw/bin:/usr/sfw/bin:/usr/ccs/bin:/usr/xpg6/bin:/usr/xpg4/bin:/usr/ucb:/usr/local/bin:/usr/bin:/bin:/usr/local/GNU/bin:/usr/sfw/sbin:/usr/local/GNU/sbin:/usr/local/sbin:/usr/sbin:/sbin

_search_for_command GNUPG gpg
_search_for_command PGP pgp
_search_for_command WGET wget

mkdir -p $ARCHIVESDIR || exit 1
for archive in $ARCHIVES; do
  fn=`echo $archive | sed 's!^.*/\([^/][^/]*\)$!\1!'`
  if [ ! -f "$ARCHIVESDIR/$fn" ]; then
    if [ -z "$WGET" ]; then
      echo "ERROR: Need to fetch $fn, but wget was not found." 1>&2
      exit 1
    fi
    echo "Downloading archive $fn ..."
    $WGET -O$ARCHIVESDIR/$fn $archive || exit 1
  fi
done

ec=0
for archive in $ARCHIVESDIR/*; do
  fn=`echo $archive | sed 's!^.*/\([^/][^/]*\)$!\1!'`
  echo
  case "$fn" in
    *.tar.gz)
      continue
    ;;
    *.sha|*.sha1)
      echo "Checking SHA1 in $fn ..."
      checksum sha1sum $fn && echo "SHA1 matched archive" && continue
      echo "ERROR: SHA1 did not verify.  Make sure this archive is valid!" 1>&2
    ;;
    *.md5)
      echo "Checking MD5 in $fn ..."
      checksum md5sum $fn && echo "MD5 matched archive" && continue
      echo "ERROR: Checksum did not verify.  Make sure this archive is valid!" 1>&2
    ;;
    *.asc|*.sig)
      echo "Checking signature in $fn ..."
      archfn=`echo $fn | sed 's!\.\(asc\|sig\)$!!'`
      if [ -n "$GNUPG" ]; then
        $GNUPG --keyserver $KEYSERVER --keyserver-options auto-key-retrieve --verify $ARCHIVESDIR/$fn && echo "Signature matched archive" && continue
      elif [ -n "$PGP" ]; then
        $PGP $ARCHIVESDIR/$fn $ARCHIVESDIR/$archfn && echo "Signature matched archive" && continue
      else
        echo "ERROR: Neither gnupg nor pgp could be found to verify $archfn" 1>&2
      fi
      echo "ERROR: Could not verify signature.  Make sure this archive is valid!" 1>&2
    ;;
    *)
      echo "ERROR: Unknown archive file type: $fn" 1>&2
      exit 1
    ;;
  esac
  ec=1
  echo
  echo "Press Enter to continue or Ctrl+C to stop now ..."
  read PRESSENTER
done

if [ "$ec" -ne 0 ]; then
  cat <<EOT
!!! WARNING: Not all of the archive signatures could be verified. !!!
  
It is recommended that you quit now with Ctrl+C.  However, if you must
go on, then press ENTER.
EOT
  read PRESSENTER
  ec=0
fi

exit $ec
