#!/bin/sh
#
# This script will build the support libs
#

### Make options for a parallel build (faster on multi-processor systems)
#PARALLELMAKE=-j3

###########################################################################

### What are we named and where are we located
SCRIPT=`basename $0`
SCRIPTDIR=`echo $0 | sed 's!'/$SCRIPT'$!!'`

# Sanity Check
if [ -z "$SCRIPTDIR" ]; then
  echo "ERROR: Could not determine script dir for $0" 1>&2
  exit 1
fi

# Need the full dir
case "$SCRIPTDIR" in
/*) ;; # Already a full path
*)
  testdir=`pwd`/$SCRIPTDIR
  if [ ! -f "$testdir/$SCRIPT" ]; then
    testdir=`which $SCRIPT`
    if [ ! -f "$testdir/$SCRIPT" ]; then
      echo "ERROR: Could not determine full path to $SCRIPT" 1>&2
      exit 1
    fi
  fi
  SCRIPTDIR=$testdir
;;
esac


### Setup the dir structure
ARCHIVESDIR=$SCRIPTDIR/archives
BUILDDIR=$SCRIPTDIR/build
INSTALLDIR=$SCRIPTDIR/install

mkdir -p $BUILDDIR || exit 1
mkdir -p $INSTALLDIR || exit 1

. $SCRIPTDIR/common.sh

### Fake GNU tar for extracting with GNU zcat and native tar
_gtar() {
  opts=`echo $1 | sed 's/z//'`
  $GZCAT $2 | tar -$opts -
}


### Look for GNU tar and/or GNU zcat
SEARCH_PATH=/opt/sfw/bin:/usr/sfw/bin:/usr/ccs/bin:/usr/xpg6/bin:/usr/xpg4/bin:/usr/ucb:/usr/local/bin:/usr/bin:/bin:/usr/local/GNU/bin:/usr/sfw/sbin:/usr/local/GNU/sbin:/usr/local/sbin:/usr/sbin:/sbin

_search_for_command GTAR gtar tar
if ( ! $GTAR zcf - /dev/null > /dev/null 2>/dev/null ); then
  _search_for_command GZCAT gzcat zcat
  if [ -z "$GZCAT" ]; then
    echo "ERROR: Could not find a useable GNU (g)tar or (g)zcat in $SEARCH_PATH" 1>&2
    exit 1
  fi
  GTAR=_gtar
fi

test -d "$ARCHIVESDIR" || exit 1

### Build archives
for archive in $ARCHIVESDIR/openssl-* $ARCHIVESDIR/curl-* $ARCHIVESDIR/apr-* $ARCHIVESDIR/pcre-*; do
  cd $SCRIPTDIR

  # Extract
  case "$archive" in
    *.tar.gz)
      subdir=`echo $archive | sed 's!^.*/\([^/][^/]*\)\.tar\.gz$!\1!'`
      if [ -z "$subdir" ]; then
        echo "ERROR: Could not determine archive subdir for $archive" 1>&2
        exit 1
      fi
      subdir=$BUILDDIR/$subdir
      if [ ! -d "$subdir" ]; then
        (
          cd $BUILDDIR || exit 1
          echo "Extracting archive $archive ..."
          $GTAR zxf $archive
        )
      fi
    ;;
    *)
      continue
    ;;
  esac

  # Build
  case "$subdir" in
    *openssl*)
      OPENSSL_DIR=$INSTALLDIR/openssl
      rm -rf $OPENSSL_DIR || exit 1
      (
        cd $subdir || exit 1
        if [ ! -f ".configured" ]; then
          ./config \
            --prefix=$OPENSSL_DIR \
            --openssldir=$OPENSSL_DIR \
            no-shared \
            no-zlib \
            no-zlib-dynamic \
            no-dso \
            no-krb5 \
          || exit 1
          touch .configured
        fi
        if [ ! -f "libssl.a" -o ! -f "libcrypto.a" ]; then
            make || exit 1
        fi
				if [ ! -d "$OPENSSL_DIR" ]; then
        	make install || exit 1
				fi
      ) 2>&1 | sed 's/^/openssl: /' 1>&2
    ;;
    *curl*)
      CURL_DIR=$INSTALLDIR/curl
      rm -rf $CURL_DIR || exit 1
      (
        cd $subdir || exit 1
        if [ ! -f "config.status" ]; then
          ./configure \
            --prefix=$CURL_DIR \
            --disable-ares \
            --disable-dict \
            --disable-file \
            --disable-ftp \
            --disable-ldap \
            --disable-manual \
            --disable-sspi \
            --disable-telnet \
            --disable-tftp \
            --enable-shared=no \
            --enable-static=yes \
            --without-gnutls \
            --without-libidn \
            --without-libssh2 \
            --without-zlib \
            --with-ssl=$OPENSSL_DIR \
          || exit 1
        fi
        if [ ! -f "lib/libcurl.la" ]; then
            make $PARALLELMAKE || exit 1
        fi
				if [ ! -d "$CURL_DIR" ]; then
        	make install || exit 1
				fi
      ) 2>&1 | sed 's/^/curl: /' 1>&2
    ;;
    *apr*)
      APR_DIR=$INSTALLDIR/apr
      rm -rf $APR_DIR || exit 1
      (
        cd $subdir || exit 1
        if [ ! -f "config.status" ]; then
          ./configure \
            --prefix=$APR_DIR \
            --enable-shared=no \
            --enable-static=yes \
            --enable-threads=yes \
            --enable-dso=no \
          || exit 1
        fi
        if [ ! -f "libapr-1.la" ]; then
          make $PARALLELMAKE || exit 1
        fi
				if [ ! -d "$APR_DIR" ]; then
        	make install || exit 1
				fi
      ) 2>&1 | sed 's/^/apr: /' 1>&2
    ;;
    *pcre*)
      PCRE_DIR=$INSTALLDIR/pcre
      rm -rf $PCRE_DIR || exit 1
      (
        cd $subdir || exit 1
        if [ ! -f "config.status" ]; then
          ./configure \
            --prefix=$PCRE_DIR \
            --enable-shared=no \
            --enable-static=yes \
            --disable-cpp \
          || exit 1
        fi
        if [ ! -f "libpcre.la" ]; then
            make $PARALLELMAKE || exit 1
        fi
				if [ ! -d "$PCRE_DIR" ]; then
        	make install || exit 1
				fi
      ) 2>&1 | sed 's/^/pcre: /' 1>&2
    ;;
  esac
done

if [ -z "$OPENSSL_DIR" -o ! -d "$OPENSSL_DIR" ]; then
  echo "ERROR: OpenSSL was not built" 1>&2
  exit 1
elif [ -z "$CURL_DIR" -o ! -d "$CURL_DIR" ]; then
  echo "ERROR: Curl was not built" 1>&2
  exit 1
elif [ -z "$APR_DIR" -o ! -d "$APR_DIR" ]; then
  echo "ERROR: APR was not built" 1>&2
  exit 1
elif [ -z "$PCRE_DIR" -o ! -d "$PCRE_DIR" ]; then
  echo "ERROR: PCRE was not built" 1>&2
  exit 1
fi

